@@ -172,7 +172,13 @@ class BaseOptionAnswer(BaseAnswerForm):
     def __init__(self, *args, **kwargs):
         super(BaseOptionAnswer, self).__init__(*args, **kwargs)
         options = self.question.parsed_options
-        choices = [(strip_tags(x), mark_safe(x)) for x in options]
+        # appendChoiceButtons in survey.js duplicates this. jQuery and django
+        # use " for html attributes, so " will mess them up.
+        choices = []
+        for x in options:
+            choices.append(
+                (strip_tags(x).replace('&amp;', '&').replace('"', "'").strip(),
+                 mark_safe(x)))
         if not self.question.required and not isinstance(self, OptionCheckbox):
             choices = [('', '---------',)] + choices
         self.fields['answer'].choices = choices
@@ -249,8 +255,10 @@ class SubmissionForm(ModelForm):
 
 
 def forms_for_survey(survey, request='testing', submission=None):
-    testing = 'testing' == request
-    session_key = "" if testing else request.session.session_key.lower()
+    testing = request == 'testing'
+    session_key = ""
+    if not testing and request.user.is_authenticated():
+        session_key = request.session.session_key.lower()
     post = None if testing else request.POST or None
     files = None if testing else request.FILES or None
     main_form = SubmissionForm(survey, data=post, files=files)
