@@ -15,11 +15,7 @@ from . import settings as local_settings
 def get_latitude_and_longitude(location):
     if geopy is None:
         raise ImportError("No module named geopy")
-    google_key = local_settings.GOOGLE_MAPS_API_KEY
-    if google_key:
-        g = geopy.geocoders.Google(google_key)
-    else:
-        g = geopy.geocoders.GeoNames(output_format='json')
+    g = geopy.geocoders.Google()
     oldstdout = sys.stdout
     try:
         sys.stdout = cStringIO.StringIO()
